require 'shellwords'
require 'escape'
require 'securerandom'

module Minicron
  # Used to interact with the crontab on hosts over an ssh connection
  class Cron
    # Initialise the cron class
    #
    # @param ssh [Minicron::Transport::SSH] instance
    def initialize(ssh)
      @ssh = ssh
    end

    # Test an SSH connection and the permissions for the crontab
    #
    # @param conn an instance of an open ssh connection
    # @return [Hash]
    def get_host_permissions(conn = nil)
      begin
        # Open an SSH connection
        conn ||= @ssh.open
      rescue
        conn = false
      end

      {
        :connect => conn != false,
      }
    end

    # Test if a host has correct permissions for minicron to operate
    #
    # @param conn an instance of an open ssh connection
    def test_host_permissions(conn)
      begin
        # Test the SSH connection first
        test = get_host_permissions(conn)
      rescue Exception => e
        raise Minicron::CronError, "Error connecting to host, reason: #{e.message}"
      end

      # Check the connection worked
      raise Minicron::CronError, "Unable to connect to host, reason: unknown" if !test[:connect]
    end

    # Build the minicron command to be used in the crontab
    #
    # @param schedule [String]
    # @param command [String]
    # @return [String]
    def build_minicron_command(schedule, command)
      # Escape the command so it will work in bourne shells
      "#{schedule} #{Escape.shell_command(['minicron', 'run', command])}"
    end

    # Build the crontab multiline string that includes all the given jobs
    #
    # @param host [Minicron::Hub::Host] a host instance with it's jobs and job schedules
    # @return [String]
    def build_crontab(host)
      # You have been warned..
      crontab = "#\n"
      crontab += "# This file was automatically generated by minicron at #{Time.now.to_s}, DO NOT EDIT manually!\n"
      crontab += "#\n\n"

      # Set the path to something sensible by default, eventually this should be configurable
      crontab += "# ENV variables\n"
      crontab += "PATH=/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin\n\n"

      # Add an entry to the crontab for each job schedule
      host.jobs.each do |job|
        crontab += "# Job ##{job.id} - #{job.name}\n"

        job.schedules.each do |schedule|
          crontab += "#{build_minicron_command(schedule.formatted, job.command)}\n"
        end
      end

      crontab
    end

    # Update the crontab on the given host
    #
    # @param host [Minicron::Hub::Host] a host instance with it's jobs and job schedules
    # @param conn an instance of an open ssh connection
    def update_crontab(host, conn = nil)
      # Open an SSH connection
      conn ||= @ssh.open

      # First check if we have the correct permissions we need
      test_host_permissions(conn)

      # Build the new crontab to set
      crontab = build_crontab(host).strip

      # Generate a temporary name for the temp crontab
      tmp_crontab_name = "/tmp/minicron_tmp_crontab_#{SecureRandom.uuid}"

      # Echo the crontab to the tmp crontab file
      conn.exec!("echo #{crontab.shellescape} > #{tmp_crontab_name}").to_s.strip

      # Check that the file has the contents we expect
      tmp_crontab = conn.exec!("cat #{tmp_crontab_name}").to_s.strip

      if tmp_crontab != crontab
        raise Minicron::CronError, "The contents of #{tmp_crontab_name} were not as expected, maybe the file couldn't be written to?"
      end

      # Install the crontab
      conn.exec!("crontab #{tmp_crontab_name}").to_s.strip

      # Check the updated crontab is set to what we expect
      updated_crontab = conn.exec!("crontab -l").to_s.strip

      if updated_crontab != crontab
        raise Minicron::CronError, 'The contents of the updated were not as expected'
      end

      # ..and finally, remove the temp crontab
      conn.exec!("rm #{tmp_crontab_name}").to_s.strip
    end
  end
end
